{% load i18n %}
<p>
  {% translate "Le sélecteur de digraphes" %}
  {{ name }}
  {% if ok %}
  {% translate "est enregistré" %}
  {% else %}
  {% translate "n'est pas enregistré ; message d'erreur :" %}
  <tt>
    {{ err }}
  </tt>
  {% endif %}
</p>
