{% load static %}
{# -*- html-mode -*- #}
{% load i18n %}
<tr class="manuel{% if p.isRendu %} rendu{% endif %}{% if p.isDegrade %} degrade{% endif %}{% if p.inventaire.Perdu %} perdu{% endif %} {% if p.est_paye %}est-paye{% endif %}"
    data-code="{{p.inventaire.id}}">
  <td class="id_manuel">
    <div>{{ p.inventaire.id }}</div>
    <button class="minibutton"
	    onclick="garde_fou('{% translate 'Attention' %}',
		     '{% translate 'Forcer le retour du livre' %}',
		     'force_retour',
		     {
		       code: {{p.inventaire.id}},
		       date: '{{ p.date_retour|date:'Y-m-d' }}'
		     }
		     )">
      <img src="{%  static '/img/logout.svg' %}" alt="rendre le manuel" title="{% translate 'rendre le manuel sans saisie du numéro' %}"/>
    </button>
  </td>
  <td class="disc">{{ p.inventaire.materiel.discipline.abrege }}</td>
  <td class="title_manuel">{{ p.inventaire.materiel.titre }}</td>
  <td class="date-pret">{{ p.date_pret|date:'j F Y' }}</td>
  <td class="date-retour">
    {% if p.isRendu %}
    {{ p.date_retour|date:'j F Y' }}
    <button class="minibutton"
	    onclick="garde_fou('Attention','Annuler le retour du livre',
		     'annule_retour',
		     {
		       code: {{p.inventaire.id}},
		       date: '{{ p.date_retour|date:'Y-m-d' }}'
		     })">
      <img src="{%  static '/img/edit-delete.svg' %}" alt="dé-rendre le manuel" title="annuler le rendu"/>
    </button>
    {% endif %}
    {% if p.inventaire.Perdu %}
    {% translate "Déclaré perdu, le" %}<br>
    {{ p.Date_Declaration_Perte|date:'j F Y' }}
    {% endif %}
  </td>
  <td class="etat-initial">
    {{ p.etat_initial }}</td>
  <td class="etat-final">
    <select onchange="change_etat_manuel_rendu(this, {{p.inventaire.id}})" data-date="{{ p.date_retour|date:'Y-m-d' }}">
      {% for id_etat, nom_etat in etats.items %}
      <option val="{{ id_etat }}"
	      {% if id_etat == etat_actuel %}selected{% endif %}>
	{{ nom_etat }}
      </option>
      {% endfor %}
    </select>
  </td>
  <td class="degradation {% if not mat.Choix_Tarif_Degradation %}hidden{% endif %}"></td>
  <td class="perdu">
    <input type="checkbox" onchange="traite_perte(this, {{ p.inventaire.id }})"
	   {% if p.inventaire.Perdu %}
	   checked="checked"
	   {% endif %} /><br>
    {% if p.inventaire.Perdu %}
    {% if p.est_paye %}
    <button class="paiement-fait" data-pret="{{ p.id }}">
      {% translate 'Livre payé le' %} {{ p.inventaire.paye.date }}
    </button>
    {% else %}
    <button data-pret="{{ p.id }}" onclick="rembourser(this)">
      {% translate 'Payer ce livre ?' %}
    </button>
    {% endif %}
    {% endif %}
  </td>
  <td class="remise{% if not mat.Appliquer_Demi_Tarif %} hidden{% endif %}">
    <input size="2" value="0"> %
  </td>
  <td class="paiement-perte{% if not mat.Choix_Tarif_Degradation %} hidden{% endif %}">
     <input size="2" value="{{mat.Choix_Tarif_Degradation}}"> %
  </td>
</tr>
