{% load static %}
{# -*- html-mode -*- #}
{% load i18n %}
<p>
  {% translate 'On va procéder au remboursement de :' %}
  {{ pret.inventaire.materiel.titre }},
  {% translate 'prêté à' %} {{ pret.eleve.NomPrenomEleve }},
  {% translate 'le' %} {{ pret.date_pret }},
  {% translate 'déclaré perdu le' %}
  {{ pret.Date_Declaration_Perte }}
</p>
<p>
  <fieldset id="paiement">
    <legend>
      {% translate 'Paiement :' %}
    </legend>
    {% for val, mode in modes.items %}
    <input type="radio" name="mode" value="{{ val }}"
	   {% if forloop.counter == 1 %}checked="checked"{% endif %}
	   /> {{ mode }}<br>
    {% endfor %}
  </fieldset>
</p>
<p>
  {% translate 'Somme à rembourser :' %} {{ pret.tarif | floatformat:2 }}
  {% translate '€' %}
</p>
